/************************************************************************\
**  multiserv_utils.c                                                   **
**                                                                      **
**  Copyright (c) 2004 Christophe Blaess <ccb@club-internet.fr>         **
**    ---------------------------------------------------------------   **
**                                                                      **
** This program is free software; you can redistribute it and/or modify **
** it under the terms of the GNU General Public License as published by **
** the Free Software Foundation.                                        **
**                                                                      **
**  This program is distributed in the hope that it will be useful,     **
** but WITHOUT ANY WARRANTY; without even the implied warranty of       **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
** GNU General Public License for more details.                         **
**                                                                      **
** You should have received a copy of the GNU General Public License    **
** along with this program; if not, write to the Free Software          **
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA             **
** 02111-1307  USA                                                      **
**                                                                      **
**    ---------------------------------------------------------------   **
**                                                                      **
** Ce programme est libre, vous pouvez le redistribuer et/ou le modifier**
** selon les termes de la Licence Publique Gnrale GNU publie par la  **
** Free Software Foundation.                                            **
**                                                                      **
** Ce programme est distribu car potentiellement utile, mais SANS      **
** AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties  **
** de commercialisation ou d'adaptation dans un but spcifique.         **
** Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails**
**                                                                      **
** Vous devez avoir reu une copie de la Licence Publique Gnrale GNU  **
** en mme temps que ce programme ; si ce n'est pas le cas, crivez    **
** la Free Software Foundation, Inc, 59 Temple Place, Suite 330, Boston **
** MA 02111-1307, tats-Unis.                                           **
**                                                                      **
\************************************************************************/

	#include <errno.h>
	#include <netdb.h>
	#include <signal.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <unistd.h>
	#include <arpa/inet.h>
	#include <sys/socket.h>
	#include <sys/wait.h>

	static int	give_port_number (char * portname, char * protocol);
	static int	give_ip_address	(char * hostname, struct in_addr * result);

	#define BUF_LEN	4096

	int
main(int argc, char * argv[])
{
	int	udp_socket;
	struct sockaddr_in address;
	char 	buffer[BUF_LEN];
	int	n;
	fd_set	set;
	
	memset(& address, 0, sizeof(address));
	address.sin_family = AF_INET;
	if ((argc != 3)
	 || (give_ip_address (argv[1], (struct in_addr *) & (address.sin_addr)) != 0)
	 || ((address.sin_port = give_port_number(argv[2], "udp")) == 0)) {
		fprintf(stderr, "usage: %s server_ip_address server_udp_port\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	udp_socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (udp_socket < 0) {
		fprintf(stderr, "Unable to get UDP/IP socket\n");
		exit(EXIT_FAILURE);
	}
	while(1) {
		FD_ZERO(& set);
		FD_SET(STDIN_FILENO, & set);
		FD_SET(udp_socket, & set);
		n = STDIN_FILENO;
		if (udp_socket > STDIN_FILENO)
			n = udp_socket;
		n = select(n+1, & set, NULL, NULL, NULL);
		if (n <= 0)
			break;
		if (FD_ISSET(STDIN_FILENO, & set)) {
			if (fgets(buffer, BUF_LEN, stdin) == NULL)
				break;
			if (sendto(udp_socket, buffer, strlen(buffer), 0, (struct sockaddr *) & address, sizeof(struct sockaddr_in)) < 0)
				break;
		}
		if (FD_ISSET(udp_socket, & set)) {
			if ((n = recvfrom(udp_socket, buffer, BUF_LEN-1, 0, NULL, NULL)) < 0)
				break;
			buffer[n] = '\0';
			if (fprintf(stdout, "%s", buffer) <= 0)
				break;
		}
	}
	close(udp_socket);
	return (0);
}

	static int
give_port_number (char * portname, char * protocol)
{
	struct servent * serv;
	short int port = 0;
	if ((serv = getservbyname (portname, protocol)) != NULL)
		return (serv -> s_port);
	if (sscanf(portname, "%hd", & port) != 1)
		return(0);
	return (htons (port));
}	





	static int
give_ip_address	(char * hostname, struct in_addr * result)
{
	struct hostent *	host;
	if (hostname == NULL) {
		result -> s_addr = htonl (INADDR_ANY);
		return (0);
	} else {
#ifdef HAVE_INET_ATON
		if (inet_aton (hostname, result) == 0) {
#else
		if ((result -> s_addr = inet_addr (hostname)) == -1) {
#endif
			if ((host = gethostbyname (hostname)) == NULL)
				return (-1);
			result -> s_addr = ((struct in_addr *) (* host -> h_addr_list)) -> s_addr;
		}
	}
	return (0);
}

